'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.handleDeclaration = handleDeclaration;

var _cssPropertyParser = require('css-property-parser');

var _cssColorableProperties = require('../../constants/cssColorableProperties');

var _reCssExplicitDefaultingKeyword = require('../../constants/reCssExplicitDefaultingKeyword');

var _rePrefixedString = require('../../constants/rePrefixedString');

var _handleEngineTriggers = require('../properties/handleEngineTriggers');

var _handleColorable = require('../properties/handleColorable');

var _handleFonts = require('../properties/handleFonts');

var _handleTransitionsAndAnimations = require('../properties/handleTransitionsAndAnimations');

var _handleDisplay = require('../properties/handleDisplay');

var _handlePosition = require('../properties/handlePosition');

var _handleZIndex = require('../properties/handleZIndex');

var _handleFloat = require('../properties/handleFloat');

var _handleBorderRadiuses = require('../properties/handleBorderRadiuses');

var _handleLetterSpacing = require('../properties/handleLetterSpacing');

var _handlePerformanceHacks = require('../properties/handlePerformanceHacks');

var _handleFunctions = require('../handleFunctions');

var _handleUnits = require('../handleUnits');

var _handleVariables = require('../handleVariables');

var _handleVendorPrefix = require('../handleVendorPrefix');

var _countUsage = require('../../calculators/countUsage');

var _removeExtraSpaces = require('../../converters/removeExtraSpaces');

var cssFontProperties = ['font', 'font-size', 'line-height', 'font-family'];

function handleDeclaration(decl, report, options) {
	report.declarations.total++;

	var declarationByteLength = Buffer.byteLength(decl.toString(), 'utf8');
	report.declarations.length.total += declarationByteLength;

	var normalizedDecl = (0, _removeExtraSpaces.removeExtraSpaces)(decl.toString());

	if (report.declarations.length.longest < declarationByteLength) {
		report.declarations.length.longest = declarationByteLength;
		report.declarations.length.longestDeclaration = normalizedDecl;
	}

	if (report.declarations.list.includes(normalizedDecl) === false) {
		report.declarations.list.push(normalizedDecl);
	}

	if (decl.important) {
		report.declarations.important++;
	}

	var prop = decl.prop;

	if (options.properties) {
		/** Count properties excluding variables */
		if (prop.startsWith('--') === false) {
			report.properties.total++;
			(0, _countUsage.countUsage)(prop, report.properties.usage);
		}

		/** Count properties with vendor prefixes */
		if (_rePrefixedString.rePrefixedString.test(prop)) {
			report.properties.prefixed++;
			(0, _handleVendorPrefix.handleVendorPrefix)(prop, report);

			/** Count property shorthands */
			// https://github.com/mahirshah/css-property-parser/issues/31
			var unprefixedProp = prop.replace(_rePrefixedString.rePrefixedString, '');
			if ((0, _cssPropertyParser.isShorthandProperty)(unprefixedProp)) {
				report.properties.shorthands++;
			}
		} else if ((0, _cssPropertyParser.isShorthandProperty)(prop)) {
			report.properties.shorthands++;
		}

		if (_reCssExplicitDefaultingKeyword.reCssExplicitDefaultingKeyword.test(decl.value)) {
			decl.value.match(_reCssExplicitDefaultingKeyword.reCssExplicitDefaultingKeyword).forEach(function (match) {
				report.properties.explicitDefaultingKeywords.total++;
				(0, _countUsage.countUsage)(match, report.properties.explicitDefaultingKeywords.usage);
			});
		}

		if (options.engineTriggerProperties) {
			(0, _handleEngineTriggers.handleEngineTriggers)(prop, report);
		}

		/**
   * Count property values resets via `all` property
   * https://drafts.csswg.org/css-cascade-4/#all-shorthand
   */
		if (prop === 'all') {
			report.properties.resetsViaAll++;
		}

		/**
   * Count anonymous replaced elements
   * https://developer.mozilla.org/en-US/docs/Web/CSS/Replaced_element
   */
		if (prop === 'content') {
			report.properties.anonymousReplacedElements++;
		}

		if (options.performanceHacks) {
			(0, _handlePerformanceHacks.handlePerformanceHacks)(decl, report);
		}
	}

	var shouldHandleAnyColors = options.colors || options.backgroundColors || options.allColors;

	if (shouldHandleAnyColors && _cssColorableProperties.cssColorableProperties.includes(prop)) {
		(0, _handleColorable.handleColorable)(decl, report, options);
	}

	if (prop === 'display' && options.displays) {
		(0, _handleDisplay.handleDisplay)(decl, report);
	}

	if (prop === 'position' && options.positions) {
		(0, _handlePosition.handlePosition)(decl, report);
	}

	if (prop === 'z-index' && options.zIndices) {
		(0, _handleZIndex.handleZIndex)(decl, report);
	}

	if (prop === 'float' && options.floats) {
		(0, _handleFloat.handleFloat)(decl, report);
	}

	if (prop.startsWith('--') === false && prop.includes('border-') && prop.endsWith('-radius') && options.borderRadiuses) {
		(0, _handleBorderRadiuses.handleBorderRadiuses)(decl, report);
	}

	if (prop === 'letter-spacing' && options.letterSpacings) {
		(0, _handleLetterSpacing.handleLetterSpacing)(decl, report);
	}

	if (cssFontProperties.includes(prop) && options.fonts) {
		(0, _handleFonts.handleFonts)(decl, report);
	}

	if (options.functions) {
		(0, _handleFunctions.handleFunctions)(decl, report, options);
	}

	if ((prop.includes('transition') || prop.includes('animation')) && options.transitionsAndAnimations) {
		(0, _handleTransitionsAndAnimations.handleTransitionsAndAnimations)(decl, report);
	}

	if (options.variables) {
		(0, _handleVariables.handleVariables)(decl, report);
	}

	if (options.units) {
		(0, _handleUnits.handleUnits)(decl, report, options);
	}
}