'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.handleFunctions = handleFunctions;

var _cssFilterFunctions = require('../../constants/cssFilterFunctions');

var _reCssFunction = require('../../constants/reCssFunction');

var _reGradient = require('../../constants/reGradient');

var _reCubicBezier = require('../../constants/reCubicBezier');

var _reUrlFunctionWithArg = require('../../constants/reUrlFunctionWithArg');

var _reImageDataUri = require('../../constants/reImageDataUri');

var _rePrefixedString = require('../../constants/rePrefixedString');

var _countUsage = require('../../calculators/countUsage');

var _removeExtraSpaces = require('../../converters/removeExtraSpaces');

var _isValidCubicBezierArgs = require('../../predicates/isValidCubicBezierArgs');

var _handleVendorPrefix = require('../handleVendorPrefix');

function countFunctions(decl, report, options) {
	decl.value.match(_reCssFunction.reCssFunction).map(function (func) {
		return func.slice(0, func.indexOf('('));
	}).forEach(function (func) {
		report.functions.total++;
		(0, _countUsage.countUsage)(func, report.functions.usage);

		if (_rePrefixedString.rePrefixedString.test(func)) {
			report.functions.prefixed++;
			(0, _handleVendorPrefix.handleVendorPrefix)(func, report);
		}

		if (_cssFilterFunctions.cssFilterFunctions.includes(func) && options.filters) {
			report.filters.total++;
			(0, _countUsage.countUsage)(func, report.filters.usage);
		}
	});
}

function countDataUris(decl, report) {
	decl.value.match(_reUrlFunctionWithArg.reUrlFunctionWithArg)
	/* eslint-disable-next-line arrow-body-style */
	.map(function (func) {
		return (0, _removeExtraSpaces.removeExtraSpaces)(func).replace(/^url\(['"]?/g, '').replace(/['"]?\)/g, '');
	}).forEach(function (urlArg) {
		if (urlArg.match(_reImageDataUri.reImageDataUri) !== null) {
			report.dataUris.total++;

			var dataUriByteLength = Buffer.byteLength(urlArg, 'utf8');
			report.dataUris.length.total += dataUriByteLength;

			if (report.dataUris.length.longest < dataUriByteLength) {
				report.dataUris.length.longest = dataUriByteLength;
				report.dataUris.length.longestDataUri = urlArg;
			}

			(0, _countUsage.countUsage)(urlArg, report.dataUris.usage);
		}
	});
}

function countGradients(decl, report) {
	decl.value.match(_reGradient.reGradient).map(function (func) {
		return (0, _removeExtraSpaces.removeExtraSpaces)(func);
	}).forEach(function (func) {
		report.gradients.total++;
		(0, _countUsage.countUsage)(func, report.gradients.usage);
	});
}

function countCubicBeziers(decl, report, options) {
	var reportSection = decl.prop.includes('animation') ? report.animations : report.transitions;

	decl.value.match(_reCubicBezier.reCubicBezier).forEach(function (func) {
		var processedFunc = (0, _removeExtraSpaces.removeExtraSpaces)(func);
		(0, _countUsage.countUsage)(processedFunc, reportSection.timingFunctions);

		/**
   * Count invalid cubic beziers
   * https://developer.mozilla.org/en-US/docs/Web/CSS/single-transition-timing-function#Examples
   */
		if ((0, _isValidCubicBezierArgs.isValidCubicBezierArgs)(processedFunc) === false) {
			(0, _countUsage.countUsage)(processedFunc, reportSection.invalidTimingFunctions);
		}
	});
}

function handleFunctions(decl, report, options) {
	if (_reCssFunction.reCssFunction.test(decl.value)) {
		countFunctions(decl, report, options);

		if (_reUrlFunctionWithArg.reUrlFunctionWithArg.test(decl.value) && options.dataUris) {
			countDataUris(decl, report);
		}

		if (_reGradient.reGradient.test(decl.value) && options.gradients) {
			countGradients(decl, report);
		}

		if (_reCubicBezier.reCubicBezier.test(decl.value) && options.transitionsAndAnimations) {
			countCubicBeziers(decl, report, options);
		}
	}
}