'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.handleColorable = handleColorable;

var _reCssNamedColor = require('../../../constants/reCssNamedColor');

var _reCssSystemColor = require('../../../constants/reCssSystemColor');

var _reHexColor = require('../../../constants/reHexColor');

var _reHexaColor = require('../../../constants/reHexaColor');

var _reRgbColor = require('../../../constants/reRgbColor');

var _reRgbaColor = require('../../../constants/reRgbaColor');

var _reHslColor = require('../../../constants/reHslColor');

var _reHslaColor = require('../../../constants/reHslaColor');

var _reHwbColor = require('../../../constants/reHwbColor');

var _countUsage = require('../../../calculators/countUsage');

var _restoreFullHex = require('../../../converters/restoreFullHex');

var _removeExtraSpaces = require('../../../converters/removeExtraSpaces');

function countColorInSection(color, reportSection) {
	reportSection.total++;
	(0, _countUsage.countUsage)(color, reportSection.usage);
}

/**
 * https://www.w3.org/TR/css-color-3/#currentcolor
 * https://www.w3.org/TR/css-color-4/#currentcolor-color
 */
var reCurrentColor = /\bcurrent[cC]olor\b/g;
var reTransparentColor = /\btransparent\b/g;

function countKeywordColors(prop, match, report, keyword, options) {
	var reportSectionName = keyword + 'Keyword';

	if (prop.startsWith('background') && options.backgroundColors) {
		match.forEach(function (matchedKeyword) {
			countColorInSection(keyword, report.backgroundColors);
			report.backgroundColors[reportSectionName]++;
		});
	} else if (prop.startsWith('background') === false && options.colors) {
		match.forEach(function (matchedKeyword) {
			countColorInSection(keyword, report.colors);
			report.colors[reportSectionName]++;
		});
	}

	if (options.allColors) {
		match.forEach(function (matchedKeyword) {
			if (options.allColors) {
				countColorInSection(keyword, report.allColors);
				report.allColors[reportSectionName]++;
			}
		});
	}
}

function countNamedColors(prop, match, report, options) {
	if (prop.startsWith('background') && options.backgroundColors) {
		match.forEach(function (color) {
			/** CSS named color declarations are not case sensitive */
			var lowerCasedColor = color.toLowerCase();

			countColorInSection(lowerCasedColor, report.backgroundColors);
			(0, _countUsage.countUsage)(lowerCasedColor, report.backgroundColors.named);
		});
	} else if (prop.startsWith('background') === false && options.colors) {
		match.forEach(function (color) {
			/** CSS named color declarations are not case sensitive */
			var lowerCasedColor = color.toLowerCase();

			countColorInSection(lowerCasedColor, report.colors);
			(0, _countUsage.countUsage)(lowerCasedColor, report.colors.named);
		});
	}

	if (options.allColors) {
		match.forEach(function (color) {
			/** CSS named color declarations are not case sensitive */
			var lowerCasedColor = color.toLowerCase();

			countColorInSection(lowerCasedColor, report.allColors);
			(0, _countUsage.countUsage)(lowerCasedColor, report.allColors.named);
		});
	}
}

function normalizeModelColor(color) {
	if (color.startsWith('#')) {
		return color.length === 4 || color.length === 5 ? (0, _restoreFullHex.restoreFullHex)(color).toLowerCase() : color.toLowerCase();
	}

	return (0, _removeExtraSpaces.removeExtraSpaces)(color);
}

function countColorModels(prop, match, report, model, options) {
	if (prop.startsWith('background') && options.backgroundColors) {
		match.forEach(function (color) {
			var normalizedColor = normalizeModelColor(color);

			countColorInSection(normalizedColor, report.backgroundColors);
			(0, _countUsage.countUsage)(model, report.backgroundColors.models);
		});
	} else if (prop.startsWith('background') === false && options.colors) {
		match.forEach(function (color) {
			var normalizedColor = normalizeModelColor(color);

			countColorInSection(normalizedColor, report.colors);
			(0, _countUsage.countUsage)(model, report.colors.models);
		});
	}

	if (options.allColors) {
		match.forEach(function (color) {
			var normalizedColor = normalizeModelColor(color);

			countColorInSection(normalizedColor, report.allColors);
			(0, _countUsage.countUsage)(model, report.allColors.models);
		});
	}
}

function countSystemColors(prop, match, report, options) {
	if (prop.startsWith('background') && options.backgroundColors) {
		match.forEach(function (color) {
			countColorInSection(color, report.backgroundColors);
			(0, _countUsage.countUsage)(color, report.backgroundColors.system);
		});
	} else if (prop.startsWith('background') === false && options.colors) {
		match.forEach(function (color) {
			countColorInSection(color, report.colors);
			(0, _countUsage.countUsage)(color, report.colors.system);
		});
	}

	if (options.allColors) {
		match.forEach(function (color) {
			countColorInSection(color, report.allColors);
			(0, _countUsage.countUsage)(color, report.allColors.system);
		});
	}
}

function handleColorable(decl, report, options) {
	var prop = decl.prop;
	var propValue = decl.value;

	if (reCurrentColor.test(propValue)) {
		countKeywordColors(prop, propValue.match(reCurrentColor), report, 'currentColor', options);
	}

	if (reTransparentColor.test(propValue)) {
		countKeywordColors(prop, propValue.match(reTransparentColor), report, 'transparent', options);
	}

	if (_reCssNamedColor.reCssNamedColor.test(propValue)) {
		countNamedColors(prop, propValue.match(_reCssNamedColor.reCssNamedColor), report, options);
	}

	if (_reHexColor.reHexColor.test(propValue)) {
		countColorModels(prop, propValue.match(_reHexColor.reHexColor), report, 'hex', options);
	}

	if (_reHexaColor.reHexaColor.test(propValue)) {
		countColorModels(prop, propValue.match(_reHexaColor.reHexaColor), report, 'hexa', options);
	}

	if (_reRgbColor.reRgbColor.test(propValue)) {
		countColorModels(prop, propValue.match(_reRgbColor.reRgbColor), report, 'rgb', options);
	}

	if (_reRgbaColor.reRgbaColor.test(propValue)) {
		countColorModels(prop, propValue.match(_reRgbaColor.reRgbaColor), report, 'rgba', options);
	}

	if (_reHslColor.reHslColor.test(propValue)) {
		countColorModels(prop, propValue.match(_reHslColor.reHslColor), report, 'hsl', options);
	}

	if (_reHslaColor.reHslaColor.test(propValue)) {
		countColorModels(prop, propValue.match(_reHslaColor.reHslaColor), report, 'hsla', options);
	}

	if (_reHwbColor.reHwbColor.test(propValue)) {
		countColorModels(prop, propValue.match(_reHwbColor.reHwbColor), report, 'hwb', options);
	}

	if (_reCssSystemColor.reCssSystemColor.test(propValue)) {
		countSystemColors(prop, propValue.match(_reCssSystemColor.reCssSystemColor), report, options);
	}
}